#
#*===================================================================
#*
#* Licensed Materials - Property of IBM
#*
#* CA12
#*
#* (c) Copyright IBM Corp. 2014  All Rights Reserved
#*
#* US Government Users Restricted Rights - Use, duplication or
#* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#*
#*===================================================================
#
import maestro
import glob
import os

import logging

logger = logging.getLogger("CICSAPP/update_app.py")

jar_cmd = maestro.node['parts']['WAS']['installDir'] + '/IBM/WebSphere/AppServer/java/bin/jar'

def updateWSDL(file, serviceName, host, port):
    tree = maestro.xmlhandler.parse_xml(file,'wsdl')
    root = tree.getroot()
    service=root.find('{http://schemas.xmlsoap.org/wsdl/}service')
    update = False
    name = service.get('name')
    if (name==serviceName or serviceName=="*"):
        update = True
        portBinding=service.find('{http://schemas.xmlsoap.org/wsdl/}port')
        address=portBinding.find('{http://schemas.xmlsoap.org/wsdl/soap/}address')
        if address == None:
            address = portBinding.find('{http://schemas.xmlsoap.org/wsdl/soap12/}address')
        location = address.get('location')
        if location.startswith("https://"):
            prefix = "https://"
            index = 8;
        if location.startswith("http://"):
            prefix = "http://"
            index = 7;
        substring = location[index:len(location)]
        suffix = substring[substring.find('/'):len(substring)]
        location = str(prefix) + str(host) + ":" + str(port) + str(suffix)
        address.set('location', location)
        maestro.xmlhandler.write_xml(tree,file)
    return update

def updateArchive(archive, serviceName, host, port):
    if archive.endswith('.war'):
        folder = 'WEB-INF/wsdl/*.wsdl'
    elif archive.endswith('.jar'):
        folder = 'META-INF/wsdl/*.wsdl'
    else:
        return False
    maestro.trace_call(logger, ['unzip', '-oa', archive, folder])
    list = glob.glob(folder)
    update = False;
    for wsdl in list:
        if updateWSDL(wsdl, serviceName, host, port):
            maestro.trace_call(logger, [jar_cmd, '-uf', archive, wsdl])
            update = True
        os.remove(wsdl)
    return update
    
def updateApp(appFile, serviceName, host, port, persistService=True):
    
    #persist the service info into role
    if persistService:
        service = {}
        service['name'] = serviceName
        service['host'] = host
        service['port'] = port
        if not 'webservices' in maestro.role:
            maestro.role['webservices'] = []
        maestro.role['webservices'].append(service)
   
    update = False;
    if appFile.endswith('.ear'):
        # process war/jar module inside of ear
        maestro.trace_call(logger, ['unzip', '-oa', appFile, '*.war'])
        maestro.trace_call(logger, ['unzip', '-oa', appFile, '*.jar'])
        list = glob.glob('./*.war')
        list.extend(glob.glob('./*.jar'))
        for archive in list:
            if updateArchive(archive, serviceName, host, port):
                update = True;
                rc = maestro.trace_call(logger, [jar_cmd, '-uf', appFile, archive])
            os.remove(archive)
      
            
    if appFile.endswith('.war'):
        update = updateArchive(appFile, serviceName, host, port);
    return update
